import React, { FC, useState } from 'react';
import { Form, FormInstance, notification } from 'antd';
import Image from 'next/image';
import { useGoogleLogin } from '@react-oauth/google';
import AppleLogin from 'react-apple-login';
import { AUTH_USER } from '../../../constants/common';
import { getAuth, getLoginByGoogle } from '../../../actions/auth';
import { getUserData } from '../../../actions/profile';
import { CustomInput } from '../../view/CustomInput';
import { CustomInputPassword } from '../../view/CustomInputPassword';
import { FilledButton } from '../../view/FilledButton';
import { OutlinedButton } from '../../view/OutlinedButton';
import { LinkButton } from '../../view/LinkButton';
import { i18nText } from '../../../i18nKeys';

type EnterProps = {
    form: FormInstance;
    updateMode: (mode: 'enter' | 'register' | 'reset' | 'finish') => void;
    locale: string;
    handleCancel: () => void;
    updateToken: (token: string) => void;
}

export const EnterContent: FC<EnterProps> = ({
    form,
    updateMode,
    updateToken,
    locale,
    handleCancel
}) => {
    const [isLoading, setIsLoading] = useState<boolean>(false);

    const onLogin = () => {
        form.validateFields().then(() => {
            const { login, password } = form.getFieldsValue();
            setIsLoading(true);
            getAuth(locale, { login, password })
                .then((data) => {
                    if (data.jwtToken) {
                        getUserData(locale, data.jwtToken)
                            .then((profile) => {
                                localStorage.setItem(AUTH_USER, JSON.stringify(profile));
                                updateToken(data.jwtToken);
                                handleCancel();
                            })
                    }
                })
                .catch((error) => {
                    notification.error({
                        message: 'Error',
                        description: error?.response?.data?.errMessage
                    });
                })
                .finally(() => {
                    setIsLoading(false);
                })
        });
    };


    const onGoogleLogin = useGoogleLogin({
        onError: (err) => {
            notification.error({
                message: err.error,
                description: err.error_description
            });
        },
        onSuccess: (tokenResponse) => {
            setIsLoading(true);
            getLoginByGoogle(locale, tokenResponse.access_token)
                .then((data) => {
                    if (data.jwtToken) {
                        getUserData(locale, data.jwtToken)
                            .then((profile) => {
                                localStorage.setItem(AUTH_USER, JSON.stringify(profile));
                                updateToken(data.jwtToken);
                                handleCancel();
                            })
                    } else {
                        notification.error({
                            message: 'Error',
                            description: 'Access denied'
                        });
                    }
                })
                .catch((error) => {
                    const err = error?.message ? JSON.parse(error.message) : {};
                    notification.error({
                        message: 'Error',
                        description: err?.details?.errMessage || undefined
                    });
                })
                .finally(() => {
                    setIsLoading(false);
                });
        }
    });

    return (
        <>
            <Form form={form} style={{ display: 'flex', gap: 16, flexDirection: 'column' }}>
                <Form.Item
                    name="login"
                    noStyle
                    rules={[
                        {
                            type: 'email',
                            message: i18nText('errors.validEmail', locale)
                        },
                        {
                            required: true,
                            message: i18nText('error.emptyEmail', locale)
                        }
                    ]}
                >
                    <CustomInput
                        size="small"
                        placeholder="E-mail"
                        type="email"
                    />
                </Form.Item>
                <Form.Item
                    name="password"
                    noStyle
                    rules={[{
                        required: true,
                        message: i18nText('errors.emptyPass', locale)
                    }]}
                >
                    <CustomInputPassword
                        size="small"
                        placeholder={i18nText('password', locale)}
                        autoComplete="off"
                        onPressEnter={onLogin}
                    />
                </Form.Item>
            </Form>
            <FilledButton
                type="primary"
                onClick={onLogin}
                loading={isLoading}
            >
                {i18nText('enter', locale)}
            </FilledButton>
            <OutlinedButton onClick={() => updateMode('register')}>{i18nText('registration', locale)}</OutlinedButton>
            <LinkButton
                type="link"
                onClick={() => updateMode('reset')}
            >
                {`${i18nText('forgotPass', locale)}?`}
            </LinkButton>
            <span>{i18nText('or', locale)}</span>
            <AppleLogin
                clientId="bbuddy.expert"
                redirectURI="https://bbuddy.expert"
                state="bblogin"
                responseType="code"
                responseMode="query"
                render={({ onClick }) => (
                    <OutlinedButton
                        icon={<Image src="/images/apple-logo.png" height={22} width={22} alt="" />}
                        onClick={onClick}
                    >
                        {i18nText('apple', locale)}
                    </OutlinedButton>
                )}
            />
            <OutlinedButton
                icon={<Image src="/images/google-logo.png" height={20} width={20} alt="" />}
                onClick={onGoogleLogin}
            >
                {i18nText('google', locale)}
            </OutlinedButton>
        </>
    );
};
