import React, { FC, useState } from 'react';
import { Form, FormInstance, notification } from 'antd';
import Image from 'next/image';
import { useGoogleLogin } from '@react-oauth/google';
import AppleLogin from 'react-apple-login';
import { AUTH_USER } from '../../../constants/common';
import { getRegister, getRegisterByGoogle } from '../../../actions/auth';
import { getUserData, setPersonData } from '../../../actions/profile';
import { CustomInput } from '../../view/CustomInput';
import { CustomInputPassword } from '../../view/CustomInputPassword';
import { FilledButton } from '../../view/FilledButton';
import { OutlinedButton } from '../../view/OutlinedButton';
import { i18nText } from '../../../i18nKeys';

type RegisterProps = {
    form: FormInstance;
    locale: string;
    updateMode: (mode: 'enter' | 'register' | 'reset' | 'finish') => void;
    updateToken: (token: string) => void;
    handleCancel: () => void;
};

export const RegisterContent: FC<RegisterProps> = ({
    form,
    updateMode,
    locale,
    updateToken,
    handleCancel
}) => {
    const [isLoading, setIsLoading] = useState<boolean>(false);

    const onRegister = () => {
        form.validateFields().then(() => {
            const { login, password } = form.getFieldsValue();
            setIsLoading(true);
            getRegister(locale)
                .then((data) => {
                    if (data.jwtToken) {
                        setPersonData( { login, password, role: 'client', languagesLinks: [] }, locale, data.jwtToken)
                            .then((profile) => {
                                updateToken(data.jwtToken);
                                localStorage.setItem(AUTH_USER, JSON.stringify(profile.userData));
                                handleCancel();
                            })
                            .catch((error) => {
                                notification.error({
                                    message: 'Error',
                                    description: error?.response?.data?.errors?.toString()
                                });
                            });
                    }
                })
                .catch((error) => {
                    notification.error({
                        message: 'Error',
                        description: error?.response?.data?.errMessage
                    });
                })
                .finally(() => {
                    setIsLoading(false);
                })
        });
    };

    const onGoogleLogin = useGoogleLogin({
        onError: (err) => {
            notification.error({
                message: err.error,
                description: err.error_description
            });
        },
        onSuccess: (tokenResponse) => {
            setIsLoading(true);
            getRegisterByGoogle(locale, tokenResponse.access_token)
                .then((data) => {
                    if (data.jwtToken) {
                        getUserData(locale, data.jwtToken)
                            .then((profile) => {
                                localStorage.setItem(AUTH_USER, JSON.stringify(profile));
                                updateToken(data.jwtToken);
                                handleCancel();
                            })
                    }
                })
                .catch((error) => {
                    const err = error?.message ? JSON.parse(error.message) : {};
                    notification.error({
                        message: 'Error',
                        description: err?.details?.errMessage || undefined
                    });
                })
                .finally(() => {
                    setIsLoading(false);
                });
        }
    });

    return (
        <>
            <Form form={form} autoComplete="off" style={{ display: 'flex', gap: 16, flexDirection: 'column' }}>
                <Form.Item
                    name="login"
                    noStyle
                    rules={[
                        {
                            type: 'email',
                            message: i18nText('errors.validEmail', locale)
                        },
                        {
                            required: true,
                            message: i18nText('error.emptyEmail', locale)
                        }
                    ]}
                >
                    <CustomInput
                        size="small"
                        placeholder="E-mail"
                        type="email"
                    />
                </Form.Item>
                <Form.Item
                    name="password"
                    noStyle
                    rules={[{
                        required: true,
                        message: i18nText('errors.emptyPass', locale)
                    }]}
                >
                    <CustomInputPassword
                        size="small"
                        placeholder={i18nText('password', locale)}
                    />
                </Form.Item>
                <Form.Item
                    name="confirmPassword"
                    noStyle
                    dependencies={['password']}
                    hasFeedback
                    rules={[
                        {
                            required: true,
                            message: i18nText('errors.confirmPass', locale),
                        },
                        ({ getFieldValue }) => ({
                            validator(_, value) {
                                if (!value || getFieldValue('password') === value) {
                                    return Promise.resolve();
                                }
                                return Promise.reject(new Error(i18nText('errors.notMatchPass', locale)));
                            },
                        }),
                    ]}
                >
                    <CustomInputPassword
                        size="small"
                        placeholder={i18nText('confirmPass', locale)}
                    />
                </Form.Item>
            </Form>
            <FilledButton
                type="primary"
                onClick={onRegister}
                loading={isLoading}
            >
                {i18nText('registration', locale)}
            </FilledButton>
            <OutlinedButton onClick={() => updateMode('enter')}>{i18nText('enter', locale)}</OutlinedButton>
            <span>{i18nText('or', locale)}</span>
            <AppleLogin
                clientId="bbuddy.expert"
                redirectURI="https://bbuddy.expert"
                state="bbregister"
                responseType="code"
                responseMode="query"
                render={({ onClick }) => (
                    <OutlinedButton
                        icon={<Image src="/images/apple-logo.png" height={22} width={22} alt="" />}
                        onClick={onClick}
                    >
                        {i18nText('apple', locale)}
                    </OutlinedButton>
                )}
            />
            <OutlinedButton
                icon={<Image src="/images/google-logo.png" height={20} width={20} alt="" />}
                onClick={onGoogleLogin}
            >
                {i18nText('google', locale)}
            </OutlinedButton>
        </>
    );
};
